NAMES_DICTIONARY = {
    "No membrane cut, double MT": 0,
    "No membrane cut, simple MT": 1,
    "No membrane cut, no MT": 2,
    "Membrane cut, simple MT": 3,
    "Membrane cut, no MT": 4,
    "No membrane cut, double MT ?": 5,
    "No membrane cut, simple MT ?": 6,
    "No membrane cut, no MT ?": 7,
    "Membrane cut, simple MT ?": 8,
    "Membrane cut, no MT ?": 9,
}



import albumentations as A

from cnn_framework.utils.data_sets.abstract_data_set import AbstractDataSet
from cnn_framework.utils.readers.images_reader import ImagesReader
from cnn_framework.utils.data_sets.dataset_output import DatasetOutput
from cnn_framework.utils.enum import NormalizeMethods, ProjectMethods
from cnn_framework.utils.readers.utils.projection import Projection
from cnn_framework.utils.readers.utils.normalization import Normalization


class MetaphaseCnnDataSet(AbstractDataSet):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Data sources
        nb_channels = self.params.nb_modalities * self.params.nb_stacks_per_modality
        self.input_data_source = ImagesReader(
            [self.data_manager.get_microscopy_image_path],
            [[Projection(method=ProjectMethods.Channel, channels=list(range(nb_channels)))]],
            [Normalization(method=NormalizeMethods.Standardize)],
        )

    def set_transforms(self):
        height, width = self.params.input_dimensions.to_tuple(False)
        if self.is_train:
            self.transforms = A.Compose(
                [
                    A.PadIfNeeded(min_height=height, min_width=width, border_mode=0, value=0, p=1),
                    A.CenterCrop(height=height, width=width, p=1),
                    A.Rotate(border_mode=0),
                    A.HorizontalFlip(),
                    A.VerticalFlip(),
                ]
            )
        else:
            self.transforms = A.Compose(
                [
                    A.PadIfNeeded(min_height=height, min_width=width, border_mode=0, value=0, p=1),
                    A.CenterCrop(height=height, width=width, p=1),
                ]
            )

    def generate_raw_images(self, filename):
        # Output
        probabilities = self.read_output(filename)

        return DatasetOutput(
            input=self.input_data_source.get_image(filename),
            target_array=probabilities,
            additional=self.additional_data_source.get_image(filename),
        )



import numpy as np


def re_organize_channels(image: np.ndarray) -> np.ndarray:
    """
    Expect a 4 dimensions image.
    Re-organize channels to get TXYC order.
    """
    if image.ndim != 4:
        raise ValueError("Expect a 4 dimensions image.")

    # Get dimension index of smallest dimension, i.e. channels
    channels_dimension_index = np.argmin(image.shape)
    channels_dimension = image.shape[channels_dimension_index]
    if channels_dimension != 3:
        raise ValueError("Expect 3 channels: SiR-tubulin/MKLP1/Phase contrast, in that order.")
    # Put channels at the back
    image = np.moveaxis(image, channels_dimension_index, 3)

    # Get dimension index of second smallest dimension, i.e. time
    second_dimension_index = np.argsort(image.shape)[1]
    # Put time at the front
    image = np.moveaxis(image, second_dimension_index, 0)

    return image



import numpy as np

from cut_detector._widget import whole_process


# make_napari_viewer is a pytest fixture that returns a napari viewer object
def test_example_magic_widget(make_napari_viewer):
    viewer = make_napari_viewer()
    viewer.add_image(np.random.random((100, 100)))

    # Just try to open the widget
    whole_process()
